@@ -1,4 +1,4 @@
-;# $Id: lint.pl,v 3.0.1.3 1993/11/10 17:39:39 ram Exp $
+;# $Id: lint.pl,v 3.0.1.4 1994/01/24 14:28:40 ram Exp $
 ;#
 ;#  Copyright (c) 1991-1993, Raphael Manfredi
 ;#  
@@ -9,6 +9,10 @@
 ;#  of the source tree for dist 3.0.
 ;#
 ;# $Log: lint.pl,v $
+;# Revision 3.0.1.4  1994/01/24  14:28:40  ram
+;# patch16: now knows about "internal use only" variables on ?MAKE: lines
+;# patch16: now suppress "read-only var set" message when change hint
+;#
 ;# Revision 3.0.1.3  1993/11/10  17:39:39  ram
 ;# patch14: now spots stale ?M: dependencies
 ;#
@@ -69,7 +73,7 @@ sub p_make {
 	local(@ary);					# Locally defined symbols
 	local(@dep);					# Dependencies
 	local($where) = "\"$file\", line $. (?MAKE:)";
-	return unless /^[\w ]*:/;		# We only want the main dependency rule
+	return unless /^[\w+ ]*:/;		# We only want the main dependency rule
 	warn "$where: ignoring duplicate dependency listing line.\n"
 		if $makeseen{$unit}++;
 	return if $makeseen{$unit} > 1;
@@ -88,13 +92,17 @@ sub p_make {
 	undef %lintextern;
 	s|^\s*||;						# Remove leading spaces
 	chop;
-	push(@make, $_);				# Record dependencies for later perusal
 	s/:(.*)//;
 	@dep = split(' ', $1);			# Dependencies
 	@ary = split(' ');				# Locally defined symbols
+	local($nowarn);					# True when +Special is seen
 	foreach $sym (@ary) {
-		# Only symbols starting with a lowercase letter are to appear in
-		# config.sh, excepted the ones listed in Except.
+		# Ignore "internal use only" symbols as far as metalint goes.
+		# Actually, we record the presence of a '+' in front of a special
+		# unit name and use that as a hint to suppress the presence of that
+		# special unit in the defined symbol section.
+		$nowarn = ($sym =~ s/^\+//);
+
 		# We record for each shell symbol the list of units which claim to make
 		# it, so as to report duplicates.
 		if ($sym =~ /^[a-z]/ || $Except{$sym}) {
@@ -102,9 +110,11 @@ sub p_make {
 			++$defseen{$sym};
 		} else {
 			warn "$where: special unit '$sym' should not be listed as made.\n"
-				unless $sym eq $unit;
+				unless $sym eq $unit || $nowarn;
 		}
 	}
+	# Record dependencies for later perusal
+	push(@make, join(' ', @ary) . ':' . join(' ', @dep));
 	foreach $sym (@dep) {
 		if ($sym =~ /^\+[A-Za-z]/) {
 			$sym =~ s|^\+||;
@@ -455,7 +465,8 @@ sub p_body {
 					unless $lintchange{$sym};
 			} else {
 				# Record that the variable is set but not listed locally.
-				$shset{$unit} .= "$sym " unless $shset{$unit} =~ /\b$sym\b/;
+				$shset{$unit} .= "$sym " unless
+					$shset{$unit} =~ /\b$sym\b/ || $lintchange{$sym};
 			}
 		}
 	}
